/*-
 * Copyright (C)2012 @BABOLO http://www.babolo.ru/
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: transposit.h,v 1.5 2013/01/26 15:43:58 babolo Exp $
 */

/*****************************************************************************************************
 *    lnkn  prev,   lnkn  ,  prev == TRANSPOSIT_EXT    *
 *****************************************************************************************************/
typedef struct transposit_link {
    u_int32_t prev;
#   define TRANSPOSIT_EXT 0xFFFFFFFF
    u_int32_t lnkn;
} transposit_link;

/*****************************************************************************************************
 *    2x2  (src)    state:                         *
 *   0 -                                                               *
 *   1 -                                                                      *
 *****************************************************************************************************/
typedef struct transposit_2swnet {
    transposit_link    src[2];
    u_char             state;
} transposit_2swnet;

/*****************************************************************************************************
 *    num  2x2  nin   nou .                            *
 *  BLIN  flags.                                                                      *
 *  out        nou ,    .  *
 *     net,       num . *
 *****************************************************************************************************/
typedef struct transposit_2sw {
    u_int32_t          flags;
    size_t             num;
    size_t             nin;
    size_t             nou;
    transposit_2swnet *net;
    transposit_link    out[0];
} transposit_2sw;

/*****************************************************************************************************
 *      nin   nou .                           *
 *  swt       .                                             *
 *  BLIN  flags.                                                                      *
 *****************************************************************************************************/
typedef struct transposit_switch {
    u_int32_t          flags;
    size_t             nin;
    size_t             nou;
    size_t             swt[0]; /* nou     0  (nin - 1) */
} transposit_switch;

/*****************************************************************************************************
 *    .                                                          *
 *****************************************************************************************************/
struct transposit_depocodes;
typedef struct transposit_depocodes {
    struct transposit_depocodes *next;
    u_char             coden;
    u_char             codes[0];
} transposit_depocodes;

/*****************************************************************************************************
 *  -          .     *
 *****************************************************************************************************/
struct transposit_deposit;
typedef struct transposit_deposit {
    struct transposit_deposit *next;
    transposit_switch         *swc;
    transposit_depocodes      *codes;
} transposit_deposit;

transposit_2sw    *transposit_2swnew    __P((u_int32_t flags, size_t in, size_t out, size_t net));
int                transposit_2swnetset __P((transposit_2sw *sw, void *bits));
transposit_switch *transposit_2swget    __P((transposit_2sw *sw, transposit_switch **swe));
int                transposit_swcmp     __P((transposit_switch *swr0, transposit_switch *swr1));
int                transposit_swuniq    __P((transposit_deposit *sdp, void *codes, u_char coden));
int                transposit_swsort    __P((transposit_deposit **sdp));
u_int64_t          transposit_pack      __P((transposit_switch *swr, int width));
void               transposit_swprint   __P((transposit_switch *swr, FILE *o));
void               transposit_deprint   __P((transposit_deposit *sdp, FILE *o));
void               transposit_2swdump   __P((transposit_2sw *sw, FILE *o));
int                transposit_swdepo    __P(( transposit_deposit **sdp
                                            , transposit_switch *swr
                                            , void *codes
                                            , u_char coden
                                            ))
;
int                transposit_2swconn   __P(( transposit_2sw *sw
                                            , u_int32_t ousw
                                            , u_int16_t oulnk
                                            , u_int32_t insw
                                            , u_int16_t inlnk
                                            ))
;
